#pragma once
#include <iostream>
#include <fstream>
#include <set>
#include <iomanip>

class SportsComplex
{
private:
	struct Info
	{
		char client_surname[64];
		char sport_activity[64];
		char trainer_surname[64];
		char date[10];
		char time_of_beginning[5];
		int code;
		int minutes;
		double tarif;

		Info() :
			code(0),
			minutes(0),
			tarif(0.0)
		{
			client_surname[0] = '\0';
			sport_activity[0] = '\0';
			trainer_surname[0] = '\0';
			date[0] = '\0';
			time_of_beginning[0] = '\0';
		}

		Info(char client_surname_[64], char sport_activity_[64], char trainer_surname_[64],
			char date_[10], char time_of_beginning_[5], int code_, int minutes_,
			double tarif_) :
			code(code_), minutes(minutes_), tarif(tarif_)
		{
			for (int i = 0; i < strlen(client_surname_); ++i)
			{
				client_surname[i] = client_surname_[i];
			}
			client_surname[std::min(strlen(client_surname_), size_t(63))] = '\0';

			for (int i = 0; i < strlen(sport_activity_); ++i)
			{
				sport_activity[i] = sport_activity_[i];
			}
			sport_activity[std::min(strlen(sport_activity_), size_t(63))] = '\0';

			for (int i = 0; i < strlen(trainer_surname_); ++i)
			{
				trainer_surname[i] = trainer_surname_[i];
			}
			trainer_surname[std::min(strlen(trainer_surname_), size_t(63))] = '\0';

			for (int i = 0; i < strlen(date_); ++i)
			{
				date[i] = date_[i];
			}
			date[std::min(strlen(date_), size_t(9))] = '\0';

			for (int i = 0; i < strlen(time_of_beginning_); ++i)
			{
				time_of_beginning[i] = time_of_beginning_[i];
			}
			time_of_beginning[std::min(strlen(time_of_beginning_), size_t(4))] = '\0';
		}

		Info(const Info& other_info) 
		{
			for (int i = 0; i < 64; ++i) 
			{
				if (i < strlen(other_info.client_surname))
				{
					client_surname[i] = other_info.client_surname[i];
				}
				else 
				{
					client_surname[i] = '\0';
					break;
				}
			}

			for (int i = 0; i < 64; ++i)
			{
				if (i < strlen(other_info.sport_activity))
				{
					sport_activity[i] = other_info.sport_activity[i];
				}
				else
				{
					sport_activity[i] = '\0';
					break;
				}
			}

			for (int i = 0; i < 64; ++i)
			{
				if (i < strlen(other_info.trainer_surname))
				{
					trainer_surname[i] = other_info.trainer_surname[i];
				}
				else
				{
					trainer_surname[i] = '\0';
					break;
				}
			}

			for (int i = 0; i < 10; ++i)
			{
				if (i < strlen(other_info.date))
				{
					date[i] = other_info.date[i];
				}
				else
				{
					date[i] = '\0';
					break;
				}
			}

			for (int i = 0; i < 5; ++i)
			{
				if (i < strlen(other_info.time_of_beginning))
				{
					time_of_beginning[i] = other_info.time_of_beginning[i];
				}
				else
				{
					time_of_beginning[i] = '\0';
					break;
				}
			}
			
			code = other_info.code;
			minutes = other_info.minutes;
			tarif = other_info.tarif;
		}

		Info& operator = (const Info& other_info) 
		{
		    //!!! Проверка на самоприсваивание
		    //!!! Для работы со строками есть готовые функции.
		    
			for (int i = 0; i < 64; ++i) 
			{
				if (i < strlen(other_info.client_surname))
				{
					client_surname[i] = other_info.client_surname[i];
				}
				else 
				{
					client_surname[i] = '\0';
					break;
				}
			}

			for (int i = 0; i < 64; ++i)
			{
				if (i < strlen(other_info.sport_activity))
				{
					sport_activity[i] = other_info.sport_activity[i];
				}
				else
				{
					sport_activity[i] = '\0';
					break;
				}
			}

			for (int i = 0; i < 64; ++i)
			{
				if (i < strlen(other_info.trainer_surname))
				{
					trainer_surname[i] = other_info.trainer_surname[i];
				}
				else
				{
					trainer_surname[i] = '\0';
					break;
				}
			}

			for (int i = 0; i < 10; ++i)
			{
				if (i < strlen(other_info.date))
				{
					date[i] = other_info.date[i];
				}
				else
				{
					date[i] = '\0';
					break;
				}
			}

			for (int i = 0; i < 5; ++i)
			{
				if (i < strlen(other_info.time_of_beginning))
				{
					time_of_beginning[i] = other_info.time_of_beginning[i];
				}
				else
				{
					time_of_beginning[i] = '\0';
					break;
				}
			}
			code = other_info.code;
			minutes = other_info.minutes;
			tarif = other_info.tarif;
			return *this;
		}

		bool operator == (const Info& info)
		{

			return (strcmp(client_surname, info.client_surname) == 0 && strcmp(sport_activity, info.sport_activity) == 0 
				&& strcmp(trainer_surname, info.trainer_surname) == 0 && strcmp(date, info.date) == 0 &&
				strcmp(time_of_beginning, info.time_of_beginning) == 0 && code == info.code
				&& minutes == info.minutes && tarif == info.tarif);
		}

		bool operator !=(const Info& info)
		{
			return!(*this == info);
		}

		friend bool operator < (const Info& info, const Info& other_info)
		{
			return info.minutes < other_info.minutes;
		}

		bool is_inside(const std::multiset<Info> container) //!!! Передача по значению
		{
			for (auto i = container.begin(); i != container.end(); ++i)
			{
				if (*this == *i)
				{
					return true;
				}
			}
			return false;
		}
	};

	std::multiset<Info> container;
public:
	SportsComplex(): container(std::multiset<Info>()) {}
	
	//!!! Передача по значению
	SportsComplex(std::multiset<Info> container_) : container(container_) {}

	bool operator == (const SportsComplex&);

	bool operator != (const SportsComplex& info)
	{
		return !(*this == info);
	}

	friend std::istream& operator >> (std::istream&, SportsComplex&);
	friend std::ostream& operator << (std::ostream&, const SportsComplex&);
	
	void write(std::ofstream&);
	void read(std::ifstream&);
	static void create(std::ofstream&);
	void delete_(std::ofstream&, std::ifstream&);
	void replace(std::ofstream&, std::ifstream&, SportsComplex&);
	static void to_txt(std::ofstream&, std::ifstream&);
};